<?php

interface SmartQueueProcessor {

  /**
   * @return SmartQueue
   */
  public function next();

}

class SmartQueueProcessorDefault implements SmartQueueProcessor {

  /**
   * @inherit
   */
  public function next() {
    return smartqueue_load($this->nextItem());
  }

  /**
   * Gets the next workable item from the queue.
   *
   * @param int $min
   *   The qid of the last checked queue item.
   * @return int
   *   The qid of the next queue item ready for processing.
   */
  protected function nextItem($min = NULL) {
    if (!$qid = $this->query($min)) {
      // No results.
      return;
    }

    // Attempt to acquire lock for right to process.
    $lock_name = 'smartqueue_' . $qid;

    // If unable, try again.
    if (!lock_acquire($lock_name)) {
      $qid = $this->nextItem($qid);
    }

    return $qid;
  }

  /**
   * Query the next queue item from the database.
   *
   * @param int $min
   *   The qid of the last checked queue item.
   * @return int
   *   The qid of the next queue item ready for processing.
   */
  protected function query($min = NULL) {
    $args = array();

    if ($min) {
      $query = 'SELECT qid FROM {smartqueue} WHERE qid > ? AND status = 0 LIMIT 1';
      $args[] = $min;
    } else {
      $query = 'SELECT qid FROM {smartqueue} WHERE status = 0 LIMIT 1';
    }

    return db_query($query, $args)->fetchField();
  }

}
